﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Drawing;                 //【Bitmap】【Color】
using System.Drawing.Imaging;         //【PixelFormat】
using System.Runtime.InteropServices; //【Marshal.Copy】

namespace WebCamCapture
{
    public partial class Form1 : Form
    {
        private Capture cam;
        private IntPtr m_ip = IntPtr.Zero;
        private const int VIDEODEVICE = 0;        //【ビデオ キャプチャー デバイスのデフォルト インデックス番号】
        private const int VIDEOWIDTH = 640;       //【水平解像度】【WebCamの機種に依存】
        private const int VIDEOHEIGHT = 480;      //【垂直解像度】【WebCamの機種に依存】
        private const int VIDEOBITSPERPIXEL = 24; //【１画素当りのビット数】【WebCamの機種に依存】
        private const int nFIFO = 10;          //【FIFOサイズ】
        private const int nFIFO2 = 5;          //【FIFOの中間段数】
        private int ptr1 = 0;                  //【FIFOポインタ1】
        private int ptr2 = nFIFO2;             //【FIFOポインタ2】
        private int[] FIFO = new int[nFIFO]; //【FIFO】
        private byte ThreshHold = 100; //【キャプチャ画像２値化閾値】
        private byte MaxLevel;
        private byte MinLevel;
        private int MaxXposition;
        private int MaxYposition;

        //****************************************************************
        //【Form初期化処理】
        //****************************************************************
        public Form1()
        {
            InitializeComponent();
            this.FormClosed += new FormClosedEventHandler(Form1_FormClosed); //【Form1にClosedイベントを追加】

            cam = new Capture(VIDEODEVICE, VIDEOWIDTH, VIDEOHEIGHT, VIDEOBITSPERPIXEL, pictureBox1);
        }

        //****************************************************************
        //【Form終了時処理】
        //****************************************************************
        void Form1_FormClosed(object sender, FormClosedEventArgs e)
        {
            cam.Dispose(); 
            if (m_ip != IntPtr.Zero)
            {
                Marshal.FreeCoTaskMem(m_ip);
                m_ip = IntPtr.Zero;
            }
        }

        //****************************************************************
        //【Button1クリック】Captureを行う
        //****************************************************************
        private unsafe void button1_Click(object sender, EventArgs e)
        {
            int i, j, k; byte* p; int a; int d1, d2, suma, sumb, sumc, L; int flg;
            Cursor.Current = Cursors.WaitCursor;

            if (m_ip != IntPtr.Zero) //【前回使用したバッファをリリースする】
            {
                Marshal.FreeCoTaskMem(m_ip);
                m_ip = IntPtr.Zero;
            }

            m_ip = cam.GetImageCapture(); //【イメージをキャプチャーする】
            p = (byte*)m_ip;
            k = nFIFO - 1;
            MaxLevel = 0;
            MinLevel = 255;
            L = 500;
            for (j = 0; j < cam.Height; j++)
            {
                for (i = 0; i < nFIFO; i++) { FIFO[i] = 0; }   //【FIFOの初期化】
                ptr1 = 0;
                ptr2 = nFIFO2;
                suma = sumb = sumc = 0;
                flg = 0;
                for (i = 0; i < cam.Width; i++)
                {
                    //『モノクロ（単一色）階調表現への変換』
                    //【URL】http://www.sm.rim.or.jp/~shishido/monog.html
                    //　a:輝度(0-255)＝Ｒ×0.30＋Ｇ×0.59＋Ｂ×0.11
                    a = (int)((float)(*p) * 0.11f + (float)(*(p + 1)) * 0.59f + (float)(*(p + 2)) * 0.30f);
                    if (MaxLevel < a)
                    { 
                        MaxLevel = (byte)a;
                        MaxXposition = i;
                        MaxYposition = j;
                    }
                    if (MinLevel > a) { MinLevel = (byte)a; }
                    a -= ThreshHold; //【閾値で±に割り振る】
                    //【FIFO処理】
                    if (a > 0) { a = 255; } else { a = 0; }
                    d1 = a - FIFO[ptr2];
                    d2 = FIFO[ptr2] - FIFO[ptr1];
                    FIFO[ptr1] = a;
                    ptr1++; if (ptr1 > k) { ptr1 = 0; }
                    ptr2++; if (ptr2 > k) { ptr2 = 0; }
                    //【エッジ判定】
                    suma = suma + d1; //【suma>0なら後半白】
                    sumb = sumb + d2; //【sumb<0なら前半黒】
                    sumc = sumc - d1 + d2; //【|sumc|が0付近ならエッジ】
                    if(sumc < 0) { sumc = -sumc; }
                    if      ((flg == 0) && (sumc < L) && (suma < -L) && (sumb > L))
                    { flg = 1; } //【白⇒黒】
                    else if ((flg == 2) && (sumc < L) && (suma > L) && (sumb < -L))
                    { flg = 3; } //【黒⇒白】
                //    a += ThreshHold;
                    switch(flg)
                    {
                        case 1:
                            *p = *(p + 1) = 0; *(p + 2) = 255; flg = 2; break; //【赤線】
                        case 3:
                            *p = 255; *(p + 1) = *(p + 2) = 0; flg = 0; break; //【青線】
                        default:
                            *p = *(p + 1) = *(p + 2) = (byte)a; break;
                    }
                    p += 3;
                }
            }
            ThreshHold = (byte)(((int)MaxLevel + (int)MinLevel) / 2); //【次回フレームで使う閾値】
            Bitmap b = new Bitmap(cam.Width, cam.Height, cam.Stride, PixelFormat.Format24bppRgb, m_ip);
            b.RotateFlip(RotateFlipType.RotateNoneFlipY); //【BitMapイメージを上下反転】
            pictureBox2.Image = b;
            Cursor.Current = Cursors.Default;
            label1.Text = "X = " + MaxXposition.ToString();
            label2.Text = "Y = " + MaxYposition.ToString();
        }

    }
}
